@@ -43,13 +43,13 @@ class CGSmartImage extends CGExtensions
     $this->RegisterModulePlugin(); // this is here so the module can be used from the admin.
   }
 
-  function LazyLoadAdmin() { return TRUE; }
+  function LazyLoadAdmin() { return FALSE; }
   function LazyLoadFrontend() { return FALSE; }
   function GetName() { return get_class($this); }
   function GetFriendlyName() { return $this->Lang('friendlyname');  }
-  function GetVersion() { return '1.9.4'; }
-  function MinimumCMSVersion() { return '1.10.3'; }
-  function GetDependencies() { return array('CGExtensions'=>'1.29');   }
+  function GetVersion() { return '1.10.4'; }
+  function MinimumCMSVersion() { return '1.11.3'; }
+  function GetDependencies() { return array('CGExtensions'=>'1.32');   }
   function GetAuthor() { return 'calguy1000'; }
   function GetAuthorEmail() { return 'calguy1000@gmail.com'; }
   function IsPluginModule() { return true; }
@@ -57,7 +57,8 @@ class CGSmartImage extends CGExtensions
   function HandlesEvents () { return false; }
   function HasAdmin() { return true; }
   function VisibleToAdminUser() { return $this->CheckPermission('Modify Site Preferences'); }
-  
+  function AllowAutoInstall() { return FALSE; }
+  function AllowAutoUpgrade() { return FALSE; }
   
   public function InitializeFrontend()
   {
@@ -141,6 +142,7 @@ class CGSmartImage extends CGExtensions
     $this->CreateParameter('filter_meanremoval','',$this->Lang('param_filter_meanremoval'));
     $this->CreateParameter('filter_negate','',$this->Lang('param_filter_negate'));
     $this->CreateParameter('filter_pixelate','',$this->Lang('param_filter_pixelate'));
+    $this->CreateParameter('filter_reflect','',$this->Lang('param_filter_reflect'));
     $this->CreateParameter('filter_resize','',$this->Lang('param_filter_resize'));
     $this->CreateParameter('filter_resizetofit','',$this->Lang('param_filter_resizetofit'));
     $this->CreateParameter('filter_rotate','',$this->Lang('param_filter_rotate'));
@@ -150,7 +152,7 @@ class CGSmartImage extends CGExtensions
 
   function GetHelp() 
   { 
-    return file_get_contents(dirname(__FILE__).'/help.inc');
+    return file_get_contents(__DIR__.'/help.inc');
   }
 
   function GetAdminDescription()
@@ -160,7 +162,7 @@ class CGSmartImage extends CGExtensions
   
   function GetChangeLog()
   {
-    return @file_get_contents(dirname(__FILE__).'/changelog.html.inc');
+    return @file_get_contents(__DIR__.'/changelog.html.inc');
   }
 
   function InstallPostMessage()
@@ -181,18 +183,17 @@ class CGSmartImage extends CGExtensions
     $imagsize = filesize($src);
 
     $browser = cge_utils::get_browser();
-    switch( $browser->getBrowser() )
-      {
-      case Browser::BROWSER_IE:
-	$ver = $browser->getVersion();
-	if( $ver <= 7 ) return FALSE;
-	if( $CMS_STYLESHEET && $ver < 8 ) return FALSE;
-	if( $ver == 8 && $imagsize >= 32*1024 ) return FALSE;
-	return TRUE;
+    switch( $browser->getBrowser() ) {
+    case Browser::BROWSER_IE:
+      $ver = $browser->getVersion();
+      if( $ver <= 7 ) return FALSE;
+      if( $CMS_STYLESHEET && $ver < 8 ) return FALSE;
+      if( $ver == 8 && $imagsize >= 32*1024 ) return FALSE;
+      return TRUE;
 
-      default:
-	return TRUE;
-      }
+    default:
+      return TRUE;
+    }
   }
 
   public function can_embed($src)
@@ -203,73 +204,66 @@ class CGSmartImage extends CGExtensions
     $e_sizelimit = (int)$this->GetPreference('embed_size');
     $e_types = $this->GetPreference('embed_types');
 
-    switch( $e_mode )
-      {
-      case 'smart':
-	return $this->can_smart_embed($src);
+    switch( $e_mode ) {
+    case 'smart':
+      return $this->can_smart_embed($src);
 
-      case 'smart_limited':
-	if( $e_sizelimit <= 0 || $e_sizelimit > 10000 ) return FALSE;
-	$e_sizelimit *= 1024;
-	$tmp = filesize($src);
-	if( $tmp > $e_sizelimit ) return FALSE;
-	return $this->can_smart_embed($src);
-	break;
+    case 'smart_limited':
+      if( $e_sizelimit <= 0 || $e_sizelimit > 10000 ) return FALSE;
+      $e_sizelimit *= 1024;
+      $tmp = filesize($src);
+      if( $tmp > $e_sizelimit ) return FALSE;
+      return $this->can_smart_embed($src);
+      break;
 	
-      case 'sizelimit':
-	// check the limit.
-	if( $e_sizelimit <= 0 || $e_sizelimit > 10000 ) return FALSE;
-	$e_sizelimit *= 1024;
-	
-	// get the filesize
-	$tmp = filesize($src);
-	if( $tmp > $e_sizelimit ) return FALSE;
-	return TRUE;
+    case 'sizelimit':
+      // check the limit.
+      if( $e_sizelimit <= 0 || $e_sizelimit > 10000 ) return FALSE;
+      $e_sizelimit *= 1024;
 
-      case 'type':
-	if( !$e_types ) return FALSE;
-	$types = explode(',',$e_types);
-	$ext = strrchr($file['name'],'.');
-	if( !$ext ) return FALSE;
-	foreach( $types as $type )
-	  {
-	    if( '.'.strtolower($type) == strtolower($extension) ) return TRUE;
-	  }
-	return FALSE;
-	break;
+      // get the filesize
+      $tmp = filesize($src);
+      if( $tmp > $e_sizelimit ) return FALSE;
+      return TRUE;
 
-      case 'none':
-      default:
-	return FALSE;
+    case 'type':
+      if( !$e_types ) return FALSE;
+      $types = explode(',',$e_types);
+      $ext = strrchr($file['name'],'.');
+      if( !$ext ) return FALSE;
+      foreach( $types as $type ) {
+	if( '.'.strtolower($type) == strtolower($extension) ) return TRUE;
       }
+      return FALSE;
+      break;
+
+    case 'none':
+    default:
+      return FALSE;
+    }
   }
 
   public function clear_cached_files()
   {
     $config = cmsms()->GetConfig();
-    $dir = $config['uploads_path'].'/_'.$this->GetName();
+	$cache_path = $this->GetPreference('cache_path', cms_join_path('uploads', '_'.$this->GetName()));
+	$dir = cms_join_path($config['root_path'], $cache_path);
     $age_days = (int)$this->GetPreference('cache_age');
-    if( $age_days <= 0 )
-      {
-	throw new Exception($this->Lang('error_invalid_age'));
-      }
+    if( $age_days <= 0 ) throw new Exception($this->Lang('error_invalid_age'));
 	  
     $thedate = time() - $age_days * 24 * 60 * 60;
     $n_removed = 0;
-    if( $dh = opendir($dir) ) 
-      {
-	while( ($file = readdir($dh)) !== false )
-	  {
-	    if( startswith($file,'.') ) continue; // ignore hidden files.
-	    $fn = cms_join_path($dir,$file);
-	    if( filemtime($fn) < $thedate && is_file($fn) )
-	      {
-		@unlink($fn);
-		$n_removed++;
-	      }
-	  }
-	closedir($dh);
+    if( $dh = opendir($dir) ) {
+      while( ($file = readdir($dh)) !== false ) {
+	if( startswith($file,'.') ) continue; // ignore hidden files.
+	$fn = cms_join_path($dir,$file);
+	if( filemtime($fn) < $thedate && is_file($fn) ) {
+	  @unlink($fn);
+	  $n_removed++;
+	}
       }
+      closedir($dh);
+    }
     return $n_removed;
   }
 
@@ -288,13 +282,16 @@ class CGSmartImage extends CGExtensions
   {
     $config = cmsms()->GetConfig();
     $prefix = trim($this->GetPreference('image_url_prefix'));
-    if( !$prefix || !startswith($prefix,$config['root_url']) )
-      $prefix = $config['uploads_url'];
+    if( !$prefix || !startswith($prefix,$config['root_url']) ) $prefix = $config['root_url'];
     $hascachedir = $this->GetPreference('image_url_hascachedir');
 
     $url = $prefix;
     if( !endswith($url,'/') ) $url .= '/';
-    if( !$hascachedir ) $url .= '_'.$this->GetName();
+    if( !$hascachedir ) {
+      $cache_url = $this->GetPreference('cache_path', cms_join_path('uploads', '_'.$this->GetName()));
+      $cache_url = str_replace(DIRECTORY_SEPARATOR, '/', $cache_url); // <- Ensure correct slashes for URL
+      $url .= $cache_url;
+    }
     if( !endswith($url,'/') ) $url .= '/';
     $url .= $filename;
     return $url;
@@ -302,4 +299,4 @@ class CGSmartImage extends CGExtensions
 
 } // end of class
 
-?>
+?>
\ No newline at end of file
